#include <windows.h>
#include <sqlext.h>
#include <sql.h>
#include <string.h>
#include <stdio.h>
#include "catalog.h"
#include "globals.h"

DBase Catalog::db;
Catalog::~Catalog(){ db.disconnect(); }
Catalog::Catalog(){ db.connect(); }

/// in the future mark url "searched" when updating?
int Catalog::update(URL * url){
DBData * rset;
   if(!url->dirty) return 0;
   if(url->ip){ // most likely ip have been updated & should be saved
        sprintf(query,DB_HOST_UPDATE_QUERY,url->ip,url->hostID);
        rset = db.query(query);
        if(!rset){ // if fails, url still should be saved
           err("[CATALOG] Update failed. Query: %s\n",query);
        } else{
           delete(rset);
        }
   } // if fails, url still should be saved
   sprintf(query, DB_URL_UPDATE_QUERY, url->size, url->mime, url->ID);
   rset = db.query(query);
   if(!rset){
      err("[CATALOG] Update failed. Query: %s\n",query);
   } else {
      delete(rset);
      dbg("[CATALOG] Updated HOST: %s PATH: %s\n",url->host, url->path);
   }
   return 0;
}


int Catalog::get(URL * url){
   dbg("[CATALOG] Getting a host from database...\n");
   DBData * rset = db.query(DB_NEXT_URL_QUERY);
   if(!rset){ return 1; }

   int ret = SQLMoreResults(rset->rows); // SQL is a batch - move to resultset
   if(ret==SQL_ERROR || ret==SQL_NO_DATA || ret ==SQL_INVALID_HANDLE ){
        err("[CATALOG] Cannot get resultset...\n");
        return ret;
   }
// l.link_id,l.file_size,l.mime,l.uid,l.pass,l.path,
// h.hostID,h.name,h.IP,h.port,h.protocol
   rset->bind(SQL_INTEGER,&url->ID,sizeof(url->ID)); // identity
   rset->bind(SQL_INTEGER,&url->size,sizeof(url->size));
   rset->bind(SQL_C_STINYINT,&url->mime,sizeof(url->mime));
   rset->bind(SQL_C_CHAR,url->userid,USERPASSIZE);
   rset->bind(SQL_C_CHAR,url->password,USERPASSIZE);
   rset->bind(SQL_C_CHAR,url->path,PATHSIZE);

   rset->bind(SQL_INTEGER,&url->hostID,sizeof(url->hostID));
   rset->bind(SQL_C_CHAR,url->host,HOSTSIZE);
   rset->bind(SQL_INTEGER,&url->ip,sizeof(url->ip));
   rset->bind(SQL_INTEGER,&url->port,sizeof(url->port));
   rset->bind(SQL_C_STINYINT,&url->protocol,sizeof(url->protocol));

   ret = rset->fetch(); // fill those variables with values
   delete(rset);
   if(!ret){
      dbg("[CATALOG] Got LINK: %s%s URLID: %d\n",url->host, url->path, url->ID);
   }
   return ret;
}



/* Adding new url ( break it up in 3 parts:
Part 1: so that host caching can be added, do not combine it with path insert
        insert host
        select inserted (existing) host id [ cannot be max(id) ]
Part 2: do not combine with insert in queue so we can do MIME by extension first
        inseert link
        select inserted link id
Part 3
        insert host id in queue
*/
int Catalog::add(URL * url, URL * parent){
DBData * rset;
unsigned int id;
   if(url->protocol == PROT_EMAIL){
      return AddEmail(url->path);
   }

// insert host into the hosts table
   dbg("[CATALOG] Adding host %s to database...\n",url->host);
   sprintf(query,DB_HOST_INSERT_QUERY,url->host,url->ip,url->port,url->protocol);
   rset = db.query(query);
   delete(rset);

// even if insert fails, host might exist - try to select it from hosts.
//   dbg("[CATALOG] Retrieving host id from database...\n");
   sprintf(query,DB_HOST_SELECT_INSERTED_HOST,url->host,url->port, url->protocol);
   rset = db.query(query);
   if(!rset){ return 0; } // display message in exec() only?
   rset->bind(SQL_C_ULONG,&id,sizeof(id)); // get results of the query (host id in this case)
   if( rset->fetch() ){ // will return ok or not ok
      err("[CATALOG] ERROR Retrieving host id from database...\n");
      err("[CATALOG] ERROR QUERY: %s\n",query);
      delete(rset); // not needed any more
      return 1;
   }
   delete(rset); // not needed any more

//VALUES(file_size=%u, mime=%u, uid=%s, pass=%s, path=%s, host_id=%u)
   dbg("[CATALOG] Adding path %s to database...\n",url->path);
   sprintf(query,DB_URL_INSERT_QUERY,url->size,url->mime,url->userid,url->password,url->path,id,parent->ID);
   rset = db.query(query);
   if(!rset){ // this happens if we insert duplicates
//        dbg("[CATALOG] ERROR adding path to database...\n");
        dbg("[CATALOG] ERROR QUERY: %s\n",query);
        return 0;
   }
   delete(rset); // this coudl be execd without creating recordset

   dbg("[CATALOG] Saved HOST: %s PATH: %s\n",url->host, url->path);
   if(url->mime != MIME_HTML && url->mime != MIME_UNKNOWN){ return 0; }

// insert url into queue
   rset = db.query("select max(id) from links"); // it was just inserted
   if(!rset){
        err("[CATALOG] ERROR selecting maxid from links table in database...\n");
        err("[CATALOG] ERROR QUERY: %s\n",query);
        return 0;
   }
   rset->bind(SQL_C_ULONG,&id,sizeof(id)); // get url id
   if( rset->fetch() ){
        err("[CATALOG] ERROR fetching maxid from links table in database...\n");
        err("[CATALOG] ERROR QUERY: %s\n",query);
        return 0;
   }
   delete(rset);

   sprintf(query,"insert into queue values(%d,%d)",id,0); // queue url for scan
   rset = db.query(query); // insert url id into the table
   if(!rset){ return 0; }
   delete(rset); // this coudl be execd without creating recordset
   dbg("[CATALOG] Queued url # %u\n",id);
   return 0;
}


int Catalog::AddEmail(char * email){
        dbg("[CATALOG] Adding e-mail address: %s\n",email);
        sprintf(query,"INSERT INTO email VALUES('%s')",email);
        delete db.query(query);
        return 0;
}

