#include <windows.h>
#include <sqlext.h>
#include "globals.h"
#include "db.h"

/*   SQLGetDiagRec(
     SQLSMALLINT HandleType,
     SQLHANDLE Handle,
     SQLSMALLINT RecNumber,
     SQLCHAR * Sqlstate,
     SQLINTEGER * NativeErrorPtr,
     SQLCHAR * MessageText,
     SQLSMALLINT BufferLength,
     SQLSMALLINT * TextLengthPtr )
*/
int SQLDiag(SQLSMALLINT handtype,SQLHANDLE hand){
static char msg[1024],state[6];
SQLSMALLINT size;
SQLINTEGER err;
        SQLRETURN ret = SQLGetDiagRec(handtype,hand,1,state,&err,msg,sizeof(msg),&size);
        if(ret==SQL_SUCCESS){
                dbg("[DB] ERRROR=%d STATE=%d %s\n",err,state,msg);
        }else{
                dbg("[DB] ERROR: could not display diagnostics\n");
        }
        return err;
}



//########################### DBData ####################################
DBData::DBData(SQLHANDLE con){
        // if fails "rows" will be set to SQL_NULL_HSTMT
        SQLRETURN ret = SQLAllocHandle(SQL_HANDLE_STMT,con,&rows);
        if(ret == SQL_INVALID_HANDLE || ret == SQL_ERROR){
        // donno if rows get set on SQL_ERROR only
                err("[DB] Could not allocate statement handle.");
                rows = SQL_NULL_HSTMT;
        }
        next_col=1;
}


DBData::~DBData(){
        if(rows!=SQL_NULL_HSTMT){ SQLFreeHandle(SQL_HANDLE_STMT,rows); }
}


int DBData::bind(int type, void * var, int size){
static long retsize; // we're discarding it
   SQLRETURN ret = SQLBindCol(rows,next_col,type,var,size,&retsize);
   if(ret!=SQL_SUCCESS && ret !=SQL_SUCCESS_WITH_INFO){
        err("[DB] Failed to bind column %d\n",next_col);
        SQLDiag(SQL_HANDLE_STMT,rows);
        return 1;
   }
   next_col++; // keep the # of the next row
   return 0;
}

int DBData::fetch(){
SQLRETURN ret;
   do{
      ret = SQLFetch(rows);
   }while(ret == SQL_STILL_EXECUTING);
   switch(ret){
        case SQL_SUCCESS:
        case SQL_SUCCESS_WITH_INFO:
//                dbg("[DB] Successfuly fetched data.\n");
                break;
        case SQL_NO_DATA:
                dbg("[DB] No data returned from database...\n");
                break;
        case SQL_INVALID_HANDLE:
                err("[DB] Error fetching rows (invalid handle).\n");
                SQLDiag(SQL_HANDLE_STMT,rows);
                return 1;
        case SQL_ERROR:
                err("[DB] Error fetching rows.\n");
                SQLDiag(SQL_HANDLE_STMT,rows);
                return 1;
   }
   return 0;
}

//############################## DBase ####################################
int DBase::connected; // connection counter (++ on connect,-- on disconect)
SQLHANDLE DBase::env; // one per program
SQLHANDLE DBase::con;

int DBase::disconnect(){
        connected--;
        if(!connected){ return SQLDisconnect(con); }
        else{ return 0; }
}

DBase::~DBase(){
        if(connected>0) SQLDisconnect(con);
        SQLFreeHandle(SQL_HANDLE_DBC,con);
        SQLFreeHandle(SQL_HANDLE_ENV,env);
}


DBase::DBase(){
SQLRETURN ret;
        AddDsn();
        if(env){ return; }  // env and con were already initialized
        ret = SQLAllocHandle(SQL_HANDLE_ENV,NULL,&env);
        if(ret == SQL_ERROR){
                err("[DB] Failed to allocate environment handle.\n");
                env = NULL; // env is static and has to be NULL not ODBC_???
                return;
        }
        ret = SQLSetEnvAttr(env, SQL_ATTR_ODBC_VERSION, (void*)SQL_OV_ODBC3, 4);
        if(ret == SQL_ERROR){ // || ret == SQL_INVALID_HANDLE){
                err("[DB] Failed to set environment attributes.\n");
                SQLDiag(SQL_HANDLE_ENV,env);
        }
        ret = SQLAllocHandle(SQL_HANDLE_DBC,env,&con);
        if(ret == SQL_ERROR || ret == SQL_INVALID_HANDLE){
                err("[DB] Failed to allocate connection handle.\n");
                SQLDiag(SQL_HANDLE_ENV,env);
                SQLFreeHandle(SQL_HANDLE_ENV,env); // no need for that any more
                env = NULL;
                return;
        }
}


DBData * DBase::query(char * sql){
SQLRETURN ret;
        DBData * data = new DBData(con); // this allocates the statement handle
        if(!data) return NULL;
        ret = SQLExecDirect(data->rows,sql,strlen(sql));
        if(ret != SQL_SUCCESS && ret != SQL_SUCCESS_WITH_INFO){
//           dbg("[DB] Query error\n");
                   SQLDiag(SQL_HANDLE_STMT,data->rows);
           delete(data);
           return NULL;
        }
        // get # of columns to be used in bind?
        return data;
}


int DBase::connect(){
SQLRETURN ret;
        if(!env){
                err("[DB] Could not connect - Handles are not allocated.\n");
                return 1;
        }
        if(connected){ connected++; return 0;} // already connected by other class
        GetPrivateProfileString("DataBase","dsn","=",dsn,sizeof(dsn),INIFILE);
        GetPrivateProfileString("DataBase","dsnUser","=",dsnUser,sizeof(dsnUser),INIFILE);
        GetPrivateProfileString("DataBase","dsnPass","=",dsnPass,sizeof(dsnPass),INIFILE);
        if(*dsn=='='|| *dsnUser=='=' || *dsnPass=='='){
            err("[DB] Error loading values from ini file.\n");
        }
        ret = SQLConnect(con,dsn,strlen(dsn),dsnUser,strlen(dsnUser),dsnPass,strlen(dsnPass));
        if(ret != SQL_SUCCESS && ret != SQL_SUCCESS_WITH_INFO){
           err("[DB] Unable to connect to database.\n");
                   SQLDiag(SQL_HANDLE_DBC,con);
           return 3;
        }
        connected++;
        return 0;
}


// this will add a new Data Source Name (DSN) to ODBC Data Sources
int DBase::AddDsn(){
//        GetPrivateProfileString("spider","dsn","=",dsn,sizeof dsn,INIFILE);
//      SQLCreateDataSource(NULL,dsn); // present GUI
//      ConfigDSN();
//        SQLConfigDataSource();
}

