#include <stdio.h>
#include "net.h"
#include "globals.h" // issub()
#include "parser.h" // BUFFSIZE

Net::~Net(){ WSACleanup(); }

char * Net::download(URL * url){
   char * fname = strrchr(url->path,'/')+1;
   return fname?fname+1:url->path;
}

Net::Net(){
   bandwidth = 0;
   WSADATA wsd;
   if(WSAStartup(0x0202,&wsd)){ dbg("Error upon WSAStartup()\n"); }
}


// scan return code (local use only) figure out if request succsesful
// upon finding blank line, url->buf will point past header
// upon redirect url->buf will point to the new url
// if size is found, url->size will be set
unsigned char Net::getmime(URL * url){
static char msg [128];
char http[16];
char * end;
   url->mime=MIME_ERROR;
   int retcode=0;
   if( strchr(url->buf,' ') > url->buf+14 ){
        return 1; // prevent buffer overrrun by scanf
   }
   // this can overrun the buffers if limits are not specified!!!!!!!!
   if( sscanf(url->buf,"%14s %4d %127s", http,&retcode,msg)!= 3){
//if( sscanf(url->buf,"%* %d %127s", http,&retcode,msg)!= 2){ //skips first item
        dbg("[NET::getmime] Could not scan HTTP responce code...\n");
        return 1;
   }
   // check http if it contains string HTTP
   if(retcode > 399){ return 0; }
   if(retcode > 299){ url->mime = MIME_REDIRECT; return 0;} // redirection
   if(retcode < 200) return 0; // 200 is OK, <200 is info only
   url->mime = MIME_OTHER; // default to MIME_UNKNOWN?
   url->size=0;
   for(char * i=url->buf; *i; i++){
      if(!memcmp(i,"\r\n\r\n",4)){ // if header end (see RFC1945)
         url->buf=i+4; // show past the end of header
         return 1; // success
      }
      end=issub(i,CONTENTLEN);
      if(end){ // Content-Length: XXXXX\r\n
        i=end;
        if( sscanf(i,"%d",&url->size)!=1 ){
                url->size =0; // 0 if error
        }
      }
      end=issub(i,CONTENTSTR);
      if(end){
         i=end;
         if(issub(i,"text/html")) url->mime = MIME_HTML;
         else if(issub(i,"image/")) url->mime = MIME_IMAGE;
         else if(issub(i,"video/")) url->mime = MIME_MOVIE;
         else if(issub(i,"text/plain")) url->mime = MIME_TXT;
         else if(issub(i,"compressed/")) url->mime = MIME_ZIP; // all compressed files
         else if(issub(i,"application/")) url->mime = MIME_EXE;
         else if(issub(i,"audio/")) url->mime = MIME_MUSIC;
      }
   }
   return 0;
}


// some servers will hang and not drop connection this ensures our return
unsigned int Net::read(SOCKET sock, char * buffer, unsigned int size){
struct timeval delay;
fd_set rdset;
fd_set exset;
        delay.tv_sec = 30;         // long seconds
        delay.tv_usec = 0;        // unsigned long microseconds
        int rdsize = 0;
        dbg("[NET::read] reading(%d): ",size);
        while(rdsize < size){
                FD_ZERO(&rdset);
                FD_SET(sock, &rdset);
                FD_ZERO(&exset);
                FD_SET(sock,&exset);
                dbg("s");
                int selret = select(sock,&rdset,NULL,&exset,&delay);
                // can check for SOCKET_ERROR or 0 if timelimit expires
                if(selret == SOCKET_ERROR){
                        err("[NET::read]Socket error while selecting...\n");
                        break;
                }
                if(FD_ISSET(sock,&exset)){
                        err("[NET::read] ERROR: Socket exception...\n");
                        break;
                }
                if(!selret || !(FD_ISSET(sock, &rdset)) ){
                        dbg(" TIMEOUT\n");
                        break; // timelimit hit or no data wating
                }
                dbg(".");
                int CurrentRead = recv(sock,buffer+rdsize,size-rdsize,0);
                if(CurrentRead == SOCKET_ERROR){
                        err("[NET::read] SOCKET_ERROR while reading...\n");
                        break;
                }
                if(CurrentRead == 0){
                        dbg(" CLOSED\n");
                        break;
                }
                if(CurrentRead == WSAECONNRESET){
                        dbg(" DROPPED\n");
                        break;
                }
                rdsize += CurrentRead;
        }
        buffer[rdsize]=0;
        // if rdsize == 0 close socket locally?
        bandwidth += rdsize; // keep track of bandwidth
        return rdsize;
}


/*
The URL is defined in the HTTP standard, RFC1945, typical request:
GET /art/gallery.dll/animals?animal=dog&color=black HTTP/1.0
Connection: Keep-Alive
User-Agent: Mozilla/3.0b4Gold (WinNT; I)
Host: www.TSite.com:1024
Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, * / *

RFC suggests every line should be followed by CRLF. use \r\n instead of \n
if url->dirty is set, url will be updated in catalog.update()
*/
int Net::get(URL * url){
SOCKET sock;
struct hostent * he;
struct sockaddr_in address;
   if(url->protocol!=PROT_UNKNOWN && url->protocol != PROT_HTTP){ return 0; }
   if(url->mime!=MIME_HTML && url->mime!=MIME_UNKNOWN){ return 0; }
   url->mime = MIME_ERROR;
   sock=socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
   if(sock <= 0){ return 0; } // local problem - mime not updated
//   if(!url->ip){ // if IP is 0, translate address into IP
      he = gethostbyname(url->host);
      if(!he){ closesocket(sock); return 0; }
      memcpy(&url->ip, he->h_addr, he->h_length);
//   } // might want to add retry functionality for when using old ip
   if(!url->port){
        url->port=80;
   } // default to http
   memcpy(&address.sin_addr, &url->ip, sizeof(url->ip)); // no need to htns ip
   address.sin_port = htons(url->port); // port was scanned
   address.sin_family = AF_INET;               // ip cannot be entered by hand
   url->dirty=1; // save all info from now on
   if(connect(sock,(struct sockaddr*)&address,sizeof(address))){
        dbg("[NET] Error connecting to %s\n",url->host);
         closesocket(sock); return 0;
   }
   if(!(*url->path)) strcpy(url->path,"/index.html"); //! htm or html?
   if(!strcmp(url->path,"/") ) strcpy(url->path,"/index.html");
//! send base64 encoded uid and password here
   sprintf(url->buf,"GET %s HTTP/1.1\nHost: %s\nAccept: text/html\r\n\r\n",url->path,url->host);
   if( send(sock,url->buf,strlen(url->buf),0) == SOCKET_ERROR ){ // request file
      closesocket (sock); return 0;
   }

//   dbg("receiving: ");
   int headsize=read(sock,url->buf,HEADSIZE);
   dbg("\n[NET] Got Header SIZE %d\n",headsize);
   if(headsize<=0 || headsize==SOCKET_ERROR ){ closesocket(sock); return 0;}
   DumpHead(url->buf); // debuging only
   char * oldbuff = url->buf;
   getmime(url); // this will move url-buf just past header (start of body)
   int HeaderSize=url->buf - oldbuff;

   dbg("[NET] Parsed MIME: %s SIZE: %d\n", PrintMime(url->mime), url->size);
   if(url->mime==MIME_REDIRECT || headsize<HEADSIZE){ closesocket(sock); return 1; }
   if(url->mime!=MIME_HTML){ closesocket(sock); return 0; } // parse html only

//   dbg("[NET] Downloading the rest of the file: ");
   int bufsize = url->size ? url->size-(HEADSIZE-HeaderSize) : BUFFSIZE-HEADSIZE-1;
   int readsize = read(sock,oldbuff+HEADSIZE, bufsize);
   shutdown(sock,SD_BOTH); // avoids problems during connection teardown
   closesocket(sock);

   dbg("\n[NET] URL size: %d, content read: %d\n", url->size, HEADSIZE-HeaderSize+readsize);
   if(!url->size){
        url->size = HEADSIZE - HeaderSize + readsize;
   }
   return 1; // success
}



#define HEADDUMP "heads.txt"
int Net::DumpHead(char * buff){
        dbg("..........................................................\n");
        dbg(buff);
        dbg("\n..........................................................\n");
        return 0;
}



