#include <string.h>
#include <stdio.h>
#include "url.h"
#include "globals.h" // issub()

int URL::setuid(char * uid){
   strncpy(userid,uid,USERPASSIZE);
   userid[USERPASSIZE-1]=0; // null terminate it since strncpy might not
   return (strlen(uid) < USERPASSIZE); // did it fit?
}

int URL::setpass(char * pass){
   strncpy(password,pass,USERPASSIZE);
   password[USERPASSIZE-1]=0;
   return (strlen(pass) < USERPASSIZE);
}

int URL::sethost(char * newhost){
   strncpy(host,newhost,HOSTSIZE);
   host[HOSTSIZE-1]=0;
   return (strlen(newhost) < HOSTSIZE);
}

int URL::setpath(char * newpath){
   strncpy(path,newpath,PATHSIZE);
   path[PATHSIZE-1]=0;
   return (strlen(newpath) < PATHSIZE);
}

// returns 0 when reached buffer end, < 0 on error , > 0 found url
int URL::scan(char * url, URL * parent){ // scan url from string into structure
   if(!url || !(*url)) return -1;
   if( issub(url,"javascript:") ){
        mime = MIME_SCRIPT;
        return -1;
   }
   init(NULL,0);
   char * sp=strchr(url,':'); // start parsing protocol
   if(sp){ // if "//" exists, parse protocol and go on
      *sp=0; // null terminating it here will speedup things
      protocol =( (issub(url,"http")  )? PROT_HTTP: // someone showed me this
                  (issub(url,"ftp")   )? PROT_FTP:
                  (issub(url,"mailto"))? PROT_EMAIL:
                  (issub(url,"telnet"))? PROT_TELNET:
                  (issub(url,"file")  )? PROT_FILE:
                  (issub(url,"news")  )? PROT_NEWS:
                                        PROT_OTHER );// gopher, telnet, etc...
      url=sp+3;
   }
   if(protocol==PROT_EMAIL){
        setpath(sp+1);
        return 1;
   }
	sp=strchr(url, '@'); // is there userid:password?
	if(sp){ // if there are userid and password,
		*sp='0'; // uid:pass will be it's own string now.
		char * uid=url; // first userid
		url=sp+1; // that's where url starts

		sp=strchr(uid,':'); // find password
                if(sp){ // userid and password?
                        *sp=0; sp++;
                        setpass(sp);
                }
                setuid(uid);
	}

   if(protocol==PROT_UNKNOWN){ sp=url; }// no host just path, host has http://
   else{ sp=strchr(url,'/'); }//  get path (everything after first /)
   if(sp && *sp){
      if(*sp=='/'){ // path included?
         setpath(sp);
      } else {   // no path, just filename?
         setpath(parent->path);
         char * fileptr = strrchr(path,'/');
         if(fileptr){ strncpy(fileptr+1,sp,PATHSIZE-(fileptr-path)-1 ); }
      }
      *sp=0; // cut off path from hostname:port
      if( strrchr(path,'/') == path+strlen(path) ){ // last char in path?
         strcat(path,"/index.html"); // path and no file name
      }
   }

   sp=strchr(url,':'); // find the port #
   if(sp){ // port other than 80
      *sp=0; sp++;
      int tmport;
      int itmp=sscanf(sp,"%d",&tmport); // hope this works
      if(itmp==1 && tmport > 0 && tmport <= 0xFFFF) port=tmport;
   }
   if(strlen(url)>3){
        sethost(url);  // copy url after path was cut off
   } else { // no host specified - use parent url data
      if(!(*path)){ return 0; } // no host no path? - no URL
      sethost(parent->host);
      ip = parent->ip;
      port=parent->port;
   }
   /// if(url->mime == MIME_OTHER) try to guess mime type from extension
   dbg("[URL] Scanned HOST: %s PATH: %s PORT: %d PROTOCOL: %d\n",host,path,port,protocol);
   return 1;
}

int URL::init(char * buffer, int buffsize){
   dirty=0;
   buf=buffer;
   if(buf) *buf=0;
   size=buffsize;
   ip = 0;
   mime = MIME_UNKNOWN;
   protocol=PROT_UNKNOWN; // unknown should default to HTTP
   port = PORT_DEFAULT;
   *userid=0;
   *password=0;
   *host=0;
   strcpy(path,"/index.html");
   return 0;
}


